@@ -7,6 +7,7 @@ from gi.repository import Adw, Gdk, Gio, GLib, GObject, Gtk
 import logging
 from typing import Callable, Optional
 
+from iotas.attachment_helpers import flush_all_attachments_on_disk
 from iotas.backup_manager import BackupManager
 import iotas.const as const
 from iotas.config_manager import ConfigManager
@@ -20,15 +21,6 @@ from iotas.widgets import load_widgets
 from iotas.window import Window
 
 
-@Gtk.Template(resource_path="/org/gnome/World/Iotas/ui/keyboard_shortcuts_window.ui")
-class KeyboardShortcutsWindow(Gtk.ShortcutsWindow):
-    __gtype_name__ = "KeyboardShortcutsWindow"
-
-    def __init__(self, window: Gtk.Window) -> None:
-        super().__init__()
-        self.set_transient_for(window)
-
-
 class Application(Adw.Application):
     development_mode = const.IS_DEVEL
     application_id = const.APP_ID
@@ -47,15 +39,6 @@ class Application(Adw.Application):
 
         self.__add_cli_options()
 
-        self.__base_css_resource = "{}/ui/base_style.css".format(self.props.resource_base_path)
-        self.__light_css_resource = "{}/ui/light_style.css".format(self.props.resource_base_path)
-        self.__dark_css_resource = "{}/ui/dark_style.css".format(self.props.resource_base_path)
-        self.__high_contrast_css_resource = "{}/ui/high_contrast_style.css".format(
-            self.props.resource_base_path
-        )
-        self.__base_css_provider: Gtk.CssProvider = None
-        self.__light_css_provider: Gtk.CssProvider = None
-        self.__dark_css_provider: Gtk.CssProvider = None
         self.__index_category_label_css_provider: Gtk.CssProvider = None
         style_manager = Adw.StyleManager.get_default()
         style_manager.connect("notify::dark", lambda _o, _v: self.__apply_css())
@@ -84,6 +67,7 @@ class Application(Adw.Application):
 
         self.__sync_manager.sign_out()
         self.db.delete_all_clean_synced_notes()
+        flush_all_attachments_on_disk()
         self.__config_manager.nextcloud_username = ""
         self.__window.close()
 
@@ -114,6 +98,7 @@ class Application(Adw.Application):
     def reset_database(self) -> None:
         """Destroy the database and quit."""
         self.db_base.trash()
+        flush_all_attachments_on_disk()
         self.__config_manager.nextcloud_prune_threshold = 0
         self.quit()
 
@@ -402,11 +387,6 @@ class Application(Adw.Application):
         add_action("about", lambda _o, _p: self.__show_about_dialog())
         add_action("quit", lambda _o, _p: self.__on_quit(), "<Control>q")
         add_action("settings", lambda _o, _p: self.__show_preferences_dialog(), "<Control>comma")
-        add_action(
-            "shortcuts",
-            lambda _o, _p: KeyboardShortcutsWindow(self.__window).present(),
-            "<Control>question",
-        )
         add_action(
             "create-note",
             lambda _o, _p: self.__on_create_note(),
@@ -436,45 +416,6 @@ class Application(Adw.Application):
         self.add_window(self.__window)
 
     def __apply_css(self) -> None:
-        if self.__base_css_resource is None:
-            return
-
-        display = Gdk.Display.get_default()
-
-        if self.__base_css_provider is None:
-            self.__base_css_provider = Gtk.CssProvider()
-            self.__base_css_provider.load_from_resource(self.__base_css_resource)
-            self.__light_css_provider = Gtk.CssProvider()
-            self.__light_css_provider.load_from_resource(self.__light_css_resource)
-            self.__dark_css_provider = Gtk.CssProvider()
-            self.__dark_css_provider.load_from_resource(self.__dark_css_resource)
-            self.__high_contrast_css_provider = Gtk.CssProvider()
-            self.__high_contrast_css_provider.load_from_resource(self.__high_contrast_css_resource)
-        else:
-            Gtk.StyleContext.remove_provider_for_display(display, self.__base_css_provider)
-            Gtk.StyleContext.remove_provider_for_display(display, self.__light_css_provider)
-            Gtk.StyleContext.remove_provider_for_display(display, self.__dark_css_provider)
-            Gtk.StyleContext.remove_provider_for_display(display, self.__high_contrast_css_provider)
-
-        style_manager = Adw.StyleManager.get_default()
-
-        if style_manager.get_high_contrast():
-            Gtk.StyleContext.add_provider_for_display(
-                display,
-                self.__high_contrast_css_provider,
-                Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION + 1,
-            )
-
-        style_provider = (
-            self.__dark_css_provider if style_manager.get_dark() else self.__light_css_provider
-        )
-        Gtk.StyleContext.add_provider_for_display(
-            display, style_provider, Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION
-        )
-
-        Gtk.StyleContext.add_provider_for_display(
-            display, self.__base_css_provider, Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION
-        )
         self.__apply_index_category_label_css()
 
     def __show_about_dialog(self) -> None:
@@ -488,10 +429,22 @@ class Application(Adw.Application):
             return
 
         self.__preferences_dialog = PreferencesDialog()
+        self.__preferences_dialog.connect(
+            "start-onboarding", self.__on_preferences_dialog_start_onboarding
+        )
         self.__preferences_dialog.connect("closed", self.__on_preferences_dialog_closed)
         self.__preferences_dialog.present(self.__window)
 
+    def __on_preferences_dialog_start_onboarding(self, dialog: PreferencesDialog) -> None:
+        if not self.__preferences_dialog or not self.__window:  # mypy
+            return
+        self.__preferences_dialog.close()
+        self.__window.show_login_dialog()
+
     def __on_preferences_dialog_closed(self, _dialog: GObject.Object) -> None:
+        if not self.__preferences_dialog:  # mypy
+            return
+        self.__preferences_dialog.disconnect_by_func(self.__on_preferences_dialog_start_onboarding)
         self.__preferences_dialog = None
 
     def __create_backup(self) -> None:
