@@ -10,6 +10,7 @@ import logging
 from typing import Any, Optional
 import webbrowser
 
+from iotas.attachment import Attachment
 from iotas.category_header_bar import CategoryHeaderBar
 from iotas.category_treeview_list_store import CategoryTreeViewListStore
 from iotas.config_manager import ConfigManager, HeaderBarVisibility
@@ -25,6 +26,7 @@ from iotas.note import Note
 from iotas.outline_dialog import OutlineDialog
 from iotas.outline_generator import OutlineHeading, OutlineGenerator
 from iotas.render_search_header_bar import RenderSearchHeaderBar
+from iotas.sync_manager import SyncManager
 from iotas.text_utils import (
     parse_any_url_at_iter,
     parse_markdown_automatic_link,
@@ -54,7 +56,6 @@ class Editor(Adw.Bin):
 
     LINE_LENGTH_STEP = 50
     TOAST_DURATION = 3.0
-    FONT_SETTING_KEYS = ["monospace-font-name", "document-font-name"]
     HEADERBAR_HIDE_DELAY = 2000
     MARGIN_BELOW_CURSOR = 22
 
@@ -91,6 +92,7 @@ class Editor(Adw.Bin):
 
         self.__active = False
         self.__note: Optional[Note] = None
+        self.__note_had_edit = False
         self.__updating_from_remote = False
         self.__sync_authenticated = False
         self.__buffer_handlers: list[int] = []
@@ -98,7 +100,6 @@ class Editor(Adw.Bin):
         # Any types to accommodate for lazy loading
         self.__render_view: Optional[Any] = None
         self.__html_generator: Optional[Any] = None
-        self.__font_families: dict[str, str] = {}
         self.__focus_mode_action: Gio.SimpleAction
         self.__cancel_action: Gio.SimpleAction
         self.__previous_sessions: list[EditorSessionDetails] = []
@@ -173,31 +174,21 @@ class Editor(Adw.Bin):
             self.__refresh_line_length_from_setting,
         )
 
-        # Use system setting for monospace font family
-        self.__dbus_proxy = Gio.DBusProxy.new_for_bus_sync(
-            bus_type=Gio.BusType.SESSION,
-            flags=Gio.DBusProxyFlags.NONE,
-            info=None,
-            name="org.freedesktop.portal.Desktop",
-            object_path="/org/freedesktop/portal/desktop",
-            interface_name="org.freedesktop.portal.Settings",
-            cancellable=None,
+        self.__push_font_updates()
+        style_manager.connect(
+            "notify::monospace-font-name", lambda _o, _v: self.__push_font_updates()
+        )
+        style_manager.connect(
+            "notify::document-font-name", lambda _o, _v: self.__push_font_updates()
+        )
+        self.__config_manager.connect_changed(
+            ConfigManager.USE_MONOSPACE_FONT,
+            self.__push_font_updates,
+        )
+        self.__config_manager.connect_changed(
+            ConfigManager.MARKDOWN_USE_MONOSPACE_FONT,
+            self.__push_font_updates,
         )
-        if self.__dbus_proxy is None:
-            logging.warning("Unable to establish D-Bus proxy for FreeDesktop.org font setting")
-        else:
-            self.__load_font_family_from_setting()
-            self.__dbus_proxy.connect_object("g-signal", self.__desktop_setting_changed, None)
-
-        if self.__dbus_proxy is not None:
-            self.__config_manager.connect_changed(
-                ConfigManager.USE_MONOSPACE_FONT,
-                self.__load_font_family_from_setting,
-            )
-            self.__config_manager.connect_changed(
-                ConfigManager.MARKDOWN_USE_MONOSPACE_FONT,
-                self.__load_font_family_from_setting,
-            )
 
         self.__config_manager.connect_changed(
             ConfigManager.EDITOR_FORMATTING_BAR_VISIBILTY,
@@ -213,18 +204,26 @@ class Editor(Adw.Bin):
 
         self.__focus_mode_helper = FocusModeHelper(self._sourceview.get_buffer())
 
-    def setup(self, model: CategoryTreeViewListStore) -> None:
+    def setup(
+        self,
+        model: CategoryTreeViewListStore,
+        sync_manager: SyncManager,
+    ) -> None:
         """Perform initial setup.
 
         :param CategoryTreeViewListStore model: Tree model
+        :param SyncManager sync_manager: Sync manager
         """
         self.__setup_actions()
         self._category_header_bar.setup(model)
+        self.__sync_manager = sync_manager
+
         self._search_header_bar.setup(self._sourceview)
         self._render_search_header_bar.setup()
         self.__font_size_selector.setup()
         self.__refresh_line_length_from_setting()
         self._formatting_header_bar.setup(self._sourceview)
+
         window = self.get_root()
         window.connect("notify::fullscreened", self.__on_window_fullscreened)
         self._sourceview.setup()
@@ -236,6 +235,7 @@ class Editor(Adw.Bin):
         :param EditorSessionDetails restore_session: Session to restore, optional
         """
         self.current_note = note
+        self.note_had_edit = False
         buffer = self._sourceview.get_buffer()
         for handler_id in self.__buffer_handlers:
             buffer.disconnect(handler_id)
@@ -432,6 +432,18 @@ class Editor(Adw.Bin):
         else:
             return None
 
+    def show_downloaded_attachment_in_render(self, attachment: Attachment) -> None:
+        """Update render to display newly available attachment.
+
+        :param Attachment attachment: Attachment to show
+        """
+        if self.__note is None or attachment.note_id != self.__note.id:
+            return
+        if self._render_edit_stack.get_visible_child() != self.__render_view:
+            return
+        logging.debug(f"Attachment '{attachment.filename}' downloaded")
+        self.__render_view.show_attached_image(attachment)
+
     @GObject.Property(type=bool, default=False)
     def active(self) -> bool:
         return self.__active
@@ -463,6 +475,22 @@ class Editor(Adw.Bin):
         # Using the editor having an active note to determine if
         return self.__note is not None
 
+    @GObject.Property(type=bool, default=False)
+    def note_had_edit(self) -> bool:
+        """Get whether note was modified during session.
+
+        Determines whether excerpt will later be updated.
+
+        :returns: Whether modified
+        :rtype: bool
+        """
+        return self.__note_had_edit
+
+    @note_had_edit.setter
+    def set_note_had_edit(self, value: bool) -> None:
+        """Set whether note was modified during session."""
+        self.__note_had_edit = value
+
     @Gtk.Template.Callback()
     def _on_mouse_back_button_click(
         self, _gesture: Gtk.GestureClick, _n_press: int, _x: float, _y: float
@@ -746,6 +774,7 @@ class Editor(Adw.Bin):
             if self.__note.title_is_top_line:
                 self.__note.title_is_top_line = False
             self.__note.flag_changed()
+            self.note_had_edit = True
             # Always called on UI thread
             self.emit("note-modified", self.__note)
         self._headerbar_stack.set_visible_child(self._main_header_bar)
@@ -769,7 +798,12 @@ class Editor(Adw.Bin):
             self.__initialise_html_generator()
             self.__push_font_updates()
 
-        dialog = ExportDialog(self.__note, self.__html_generator, self.__ensure_webkit_initialised)
+        dialog = ExportDialog(
+            self.__note,
+            self.__html_generator,
+            self.__sync_manager,
+            self.__ensure_webkit_initialised,
+        )
         dialog.present(self)
 
     def __on_detect_syntax_changed(self) -> None:
@@ -934,6 +968,7 @@ class Editor(Adw.Bin):
         if self.__note.title_is_top_line:
             self.__note.update_title_from_top_line()
         self.__note.flag_changed()
+        self.note_had_edit = True
 
         # Maintain a margin between the cursor and the bottom of the window
         self.__check_and_scroll_for_cursor()
@@ -964,6 +999,10 @@ class Editor(Adw.Bin):
         self.__show_toast(_("Line length limit disabled"))
 
     def __show_toast(self, msg: str) -> None:
+        if not self.active:
+            logging.debug(f"Not showing toast as editor inactive: {msg}")
+            return
+
         toast = Adw.Toast.new(msg)
         toast.set_priority(Adw.ToastPriority.HIGH)
         toast.set_timeout(self.TOAST_DURATION)
@@ -978,7 +1017,7 @@ class Editor(Adw.Bin):
 
         if self.__html_generator is None:
             self.__initialise_html_generator()
-            assert self.__html_generator
+            assert self.__html_generator  # mypy
 
         app = Gio.Application.get_default()
         track_timing = app.debug_session or app.development_mode
@@ -1054,7 +1093,7 @@ class Editor(Adw.Bin):
 
             GLib.idle_add(delayed_editor_scroll_and_cursor_reset)
             if self._render_edit_stack.get_visible_child() == self.__render_view:
-                self.__render_view.render_retaining_scroll(self.__note, None)
+                self.__render_view.render_retaining_scroll(self.__note, export_format=None)
             self.__updating_from_remote = False
             self.set_sensitive(True)
 
@@ -1214,74 +1253,30 @@ class Editor(Adw.Bin):
         length_for_view = -1 if length == self.__config_manager.line_length_max else length
         self._sourceview.line_length = length_for_view
 
-    def __desktop_setting_changed(
-        self, _sender_name: str, _signal_name: str, _parameters: str, data: GLib.Variant
-    ) -> None:
-        if _parameters != "SettingChanged":
-            return
-        (path, setting_name, value) = data
-        if path == "org.gnome.desktop.interface" and setting_name in self.FONT_SETTING_KEYS:
-            if value.strip() != "":
-                font_description = Pango.font_description_from_string(value)
-                self.__font_families[setting_name] = font_description.get_family()
-                self.__push_font_updates()
-
-    def __process_desktop_settings(self, variant: GLib.Variant) -> None:
-        if variant.get_type_string() != "(a{sa{sv}})":
-            return
-        for v in variant:
-            for key, value in v.items():
-                if key == "org.gnome.desktop.interface":
-                    for font_key in self.FONT_SETTING_KEYS:
-                        if font_key in value:
-                            font_description = Pango.font_description_from_string(value[font_key])
-                            self.__font_families[font_key] = font_description.get_family()
-        self.__push_font_updates()
-
     def __push_font_updates(self) -> None:
-        font_setting = self.__fetch_editor_font_setting_name()
-        if font_setting in self.__font_families:
-            family = self.__font_families[font_setting]
-            self._sourceview.update_font_family(family)
+        family = self.__get_editor_font_family()
+        self._sourceview.update_font_family(family)
 
-        if self.__render_view is not None:
-            font_setting = self.__fetch_render_font_setting_name()
-            if font_setting in self.__font_families:
-                family = self.__font_families[font_setting]
-                if self.__html_generator is not None:
-                    self.__html_generator.update_font_family(family)
+        if self.__html_generator is not None:
+            family = self.__get_render_font_family()
+            self.__html_generator.update_font_family(family)
+            if self.__render_view:
                 self.__render_view.update_style()
 
-    def __fetch_editor_font_setting_name(self):
-        return (
-            "monospace-font-name"
-            if self.__config_manager.use_monospace_font
-            else "document-font-name"
-        )
+    def __get_render_font_family(self) -> str:
+        return self.__get_font_family(self.__config_manager.markdown_use_monospace_font)
 
-    def __fetch_render_font_setting_name(self):
-        return (
-            "monospace-font-name"
-            if self.__config_manager.markdown_use_monospace_font
-            else "document-font-name"
-        )
+    def __get_editor_font_family(self) -> str:
+        return self.__get_font_family(self.__config_manager.use_monospace_font)
 
-    def __load_font_family_from_setting(self) -> None:
-        if self.__dbus_proxy is None:
-            return
-        try:
-            variant = self.__dbus_proxy.call_sync(
-                method_name="ReadAll",
-                parameters=GLib.Variant("(as)", ("org.gnome.desktop.*",)),
-                flags=Gio.DBusCallFlags.NO_AUTO_START,
-                timeout_msec=-1,
-                cancellable=None,
-            )
-        except GLib.GError as e:
-            logging.warning("Unable to access D-Bus FreeDesktop.org font family setting: %s", e)
-            return
-        self.__process_desktop_settings(variant)
-        self.__push_font_updates()
+    def __get_font_family(self, monospace: bool) -> str:
+        style_manager = Adw.StyleManager.get_default()
+        if monospace:
+            font = style_manager.props.monospace_font_name
+        else:
+            font = style_manager.props.document_font_name
+        font_description = Pango.font_description_from_string(font)
+        return font_description.get_family()
 
     def __show_header_bar(self) -> None:
         """Show the headerbar."""
@@ -1480,10 +1475,15 @@ class Editor(Adw.Bin):
 
         assert self.__render_view
 
+        if self.__sync_manager.authenticated:
+            download_func = self.__sync_manager.download_attachments
+        else:
+            download_func = None
         self.__render_view.render(
             self.__note,
-            None,
-            self.__get_scroll_position(),
+            export_format=None,
+            attachment_download_func=download_func,
+            scroll_position=self.__get_scroll_position(),
         )
         self._search_header_bar.disable_actions()
         self.__focus_mode_action.set_enabled(False)
