@@ -1,8 +1,9 @@
 from abc import ABC, abstractmethod
 
-from typing import Optional
+from typing import Callable, Optional
 
-import markdown_it
+from markdown_it.token import Token
+from markdown_it.utils import EnvType, OptionsDict
 
 from iotas.note import Note
 
@@ -18,18 +19,24 @@ class HtmlGenerator(ABC):
     def generate(
         self,
         note: Note,
+        tokens: list[Token],
+        render_func: Callable[[list[Token], OptionsDict, EnvType], str],
+        parser_options: OptionsDict,
         searching: bool,
         export_format: Optional[str],
         scroll_position: Optional[float] = None,
-    ) -> tuple[str, list[markdown_it.token.Token]]:
+    ) -> str:
         """Generator HTML for note.
 
         :param Note note: Note to render
+        :param list[Token] tokens: Parser tokens
+        :param Callable[[list[Token], OptionsDict, EnvType], str] render_func: Render function
+        :param OptionsDict parser_options: Parser options
         :param bool searching: Whether search CSS should be included
-        :param bool export_format: Export format, if using
+        :param Optional[str] export_format: Export format, if using
         :param Optional[float] scroll_position: Position to scroll to
-        :return: Generated HTML and list of parser tokens
-        :rtype: tuple[str, list[markdown_it.token.Token]]
+        :return: Generated HTML
+        :rtype: str
         """
         raise NotImplementedError()
 
