@@ -262,13 +262,15 @@ class IndexNoteList(Gtk.Box):
             listbox.select_row(previous_row)
             previous_row.grab_focus()
 
-    def restrict_for_search_by_ids(self, ids: Optional[list[int]]) -> None:
+    def restrict_for_search_by_ids(self, ids: Optional[list[int]], show_all: bool = False) -> None:
         """Restrict by identifiers while searching.
 
         :param Optional[list[int]] ids: Identifiers
+        :param bool show_all: Whether to show the full result set (instead of the most recent with
+            an expandable section)
         """
         assert self.__search_model is not None
-        if ids and len(ids) > self.__search_base_note_count:
+        if not show_all and ids and len(ids) > self.__search_base_note_count:
             self.__search_full_set = ids
             self._show_more_button.set_visible(True)
             ids = ids[: self.__search_base_note_count]
@@ -335,6 +337,22 @@ class IndexNoteList(Gtk.Box):
     def have_more_search_results(self) -> bool:
         return len(self.__search_full_set) > 0
 
+    @GObject.Property(type=bool, default=False)
+    def showing_expanded_search_set(self) -> bool:
+        """Whether search is active showing a set which has been expanded.
+
+        As in Show More has already been pressed.
+
+        :return: Whether searching and expanded
+        :rtype: bool
+        """
+        return (
+            self.searching
+            and not self._show_more_button.get_visible()
+            and self.__search_model is not None
+            and self.__search_model.get_n_items() > self.__search_base_note_count
+        )
+
     def __create_row(self, note: Note, _user_data: Any) -> Gtk.Widget:
         row = IndexRow()
         style = self.__config_manager.index_category_style
