@@ -51,7 +51,7 @@ class NextcloudLoginDialog(Adw.Dialog):
                 self._url_entry.set_text(endpoint)
             self.__attempt_login()
 
-        self.__sync_manager.connect("finished", lambda _o, _v: self.__on_initial_sync_finished())
+        self.__sync_manager.connect("finished", self.__on_initial_sync_finished)
 
         self.connect("closed", lambda _o: self.__on_close())
 
@@ -99,6 +99,8 @@ class NextcloudLoginDialog(Adw.Dialog):
             GLib.source_remove(self.__timeout_id)
             self.__timeout_id = None
 
+        self.__sync_manager.disconnect_by_func(self.__on_initial_sync_finished)
+
     def __on_login_success(self, storage_success: bool) -> None:
         if storage_success:
             if self._stack.get_visible_child() == self._loading:
@@ -113,7 +115,7 @@ class NextcloudLoginDialog(Adw.Dialog):
         else:
             self._stack.set_visible_child(self._secret_service_failure)
 
-    def __on_initial_sync_finished(self) -> None:
+    def __on_initial_sync_finished(self, _obj: SyncManager, _changes: int) -> None:
         self._stack.set_visible_child(self._finished)
         self._toolbar_view.set_reveal_top_bars(False)
 
