@@ -16,6 +16,7 @@ from ssl import SSLCertVerificationError
 
 import iotas.const as const
 
+from iotas.attachment import Attachment
 from iotas.nextcloud_sync_worker_configuration import NextcloudSyncWorkerConfiguration
 from iotas.note import Note
 from iotas.sync_result import SyncResult, ContentPushSyncResult
@@ -40,7 +41,8 @@ class NextcloudSyncWorker(GObject.Object):
     }
 
     ENDPOINT_V_1_0 = "/index.php/apps/notes/api/v1/"
-    EXPECTED_API_VERSION = "1.3"
+    ENDPOINT_V_1_4 = "/index.php/apps/notes/api/v1.4/"
+    EXPECTED_API_VERSION = "1.4"
 
     LOGIN_ENDPOINT = "/index.php/login/v2"
     CAPABILITIES_ENDPOINT = "/ocs/v2.php/cloud/capabilities"
@@ -66,6 +68,7 @@ class NextcloudSyncWorker(GObject.Object):
         self.__secret_service_failed = False
         self.authenticated = False
         self.__timeout = 10
+        self.__api_version = ""
 
         self.__server = self.__config_manager.nextcloud_endpoint
         self.__username = self.__config_manager.nextcloud_username
@@ -204,7 +207,7 @@ class NextcloudSyncWorker(GObject.Object):
                 timeout=self.__timeout,
             )
         except Exception as e:
-            logging.warning("GET request for capabilities failed: %s", e)
+            logging.warning(f"GET request for capabilities failed: {e}")
             return
 
         response = self.__parse_get_response_to_json(res, "capabilities")
@@ -231,6 +234,7 @@ class NextcloudSyncWorker(GObject.Object):
                 f"Expected API version {self.EXPECTED_API_VERSION} not found in server API "
                 f"versions ({', '.join(api_version)})"
             )
+        self.__api_version = max(api_version)
 
         GLib.idle_add(self.emit, "api-check-finished")
 
@@ -330,6 +334,23 @@ class NextcloudSyncWorker(GObject.Object):
         res = self.__request_delete(path)
         return SyncResult.from_requests_response(res)
 
+    def get_attachment(self, attachment: Attachment) -> Optional[bytes]:
+        """Fetch an attachment from the server.
+
+        :param Attachment attachment: Attachment
+        :return: Data
+        :rtype: bytes, optional
+        """
+        params = {}
+        params["path"] = attachment.path
+
+        res = self.__request_get(f"attachment/{attachment.note_remote_id}", params, self.__timeout)
+
+        if not res or res.status_code != 200:
+            return None
+        else:
+            return res.content
+
     def reset_prune_threshold(self) -> None:
         """Reset the sync threshold."""
         self.__prune_threshold = 0
@@ -370,6 +391,14 @@ class NextcloudSyncWorker(GObject.Object):
     def set_timeout(self, value: int) -> None:
         self.__timeout = value
 
+    @GObject.Property(type=str)
+    def api_version(self) -> str:
+        return self.__api_version
+
+    @api_version.setter
+    def set_api_version(self, value: str) -> None:
+        self.__api_version = value
+
     def __init_secrets(self) -> None:
         # Doing this sync fetch works around some type of libsecret flatpak issue (... so we may
         # as well use the result)
@@ -417,7 +446,7 @@ class NextcloudSyncWorker(GObject.Object):
                 timeout=timeout,
             )
         except Exception as e:
-            logging.warning("GET request failed: %s", e)
+            logging.warning(f"GET request failed: {e}")
             return None
 
     def __request_put(self, path: str, headers: dict, data: dict) -> Optional[requests.Response]:
@@ -434,10 +463,12 @@ class NextcloudSyncWorker(GObject.Object):
                 timeout=self.__timeout,
             )
         except Exception as e:
-            logging.warning("PUT request failed: %s", e)
+            logging.warning(f"PUT request failed: {e}")
             return None
 
-    def __request_post(self, path: str, data: dict) -> Optional[requests.Response]:
+    def __request_post(
+        self, path: str, data: dict, files: Optional[dict] = None
+    ) -> Optional[requests.Response]:
         assert self.__username is not None
         assert self.__app_password is not None
         url = self.__get_url(path)
@@ -447,11 +478,12 @@ class NextcloudSyncWorker(GObject.Object):
                 headers=self.__headers,
                 auth=(self.__username, self.__app_password),
                 data=data,
+                files=files,
                 verify=self.verify,
                 timeout=self.__timeout,
             )
         except Exception as e:
-            logging.warning("POST request failed: %s", e)
+            logging.warning(f"POST request failed: {e}")
             return None
 
     def __request_delete(self, path: str) -> Optional[requests.Response]:
@@ -467,11 +499,12 @@ class NextcloudSyncWorker(GObject.Object):
                 timeout=self.__timeout,
             )
         except Exception as e:
-            logging.warning("DELETE request failed: %s", e)
+            logging.warning(f"DELETE request failed: {e}")
             return None
 
     def __get_url(self, path: str) -> str:
-        return self.__server + "/" + self.ENDPOINT_V_1_0 + path
+        endpoint = self.ENDPOINT_V_1_4 if "attachment" in path else self.ENDPOINT_V_1_0
+        return self.__server + "/" + endpoint + path
 
     def __store_password(self, login: str, password: str) -> bool:
         success = False
@@ -486,7 +519,7 @@ class NextcloudSyncWorker(GObject.Object):
                 None,
             )
         except Exception as e:
-            logging.error("Failed to store password: %s", e)
+            logging.error(f"Failed to store password: {e}")
         return success
 
     def __fetch_password(self) -> Optional[str]:
@@ -497,7 +530,7 @@ class NextcloudSyncWorker(GObject.Object):
             if "ServiceUnknown" in e.message:
                 self.__handle_secret_service_failure(emit=True)
             else:
-                logging.error("Failed to fetch password: %s", e)
+                logging.error(f"Failed to fetch password: {e}")
         return pwd
 
     def __handle_secret_service_failure(self, emit: bool):
@@ -518,7 +551,7 @@ class NextcloudSyncWorker(GObject.Object):
         try:
             Secret.password_clear_sync(self.SECRET_SCHEMA, self.SECRET_ATTRIBUTES, None)
         except Exception as e:
-            logging.warning("Failed to clear password: %s", e)
+            logging.warning(f"Failed to clear password: {e}")
 
     def __parse_get_response_to_json(
         self, res: Optional[requests.Response], purpose: str
