@@ -7,6 +7,8 @@ from threading import Lock, Thread
 import time
 from typing import Optional
 
+from iotas.attachment import Attachment
+from iotas.attachment_helpers import write_attachment_to_disk
 from iotas.category_manager import CategoryManager
 from iotas.config_manager import ConfigManager
 from iotas.nextcloud_sync_worker import NextcloudSyncWorker, LoginRequestResult
@@ -54,6 +56,7 @@ class SyncManager(GObject.Object):
         "remote-category-sanitised": (GObject.SignalFlags.RUN_FIRST, None, (str, str)),
         "missing-password": (GObject.SignalFlags.RUN_FIRST, None, ()),
         "notes-capability-missing": (GObject.SignalFlags.RUN_FIRST, None, ()),
+        "attachment-downloaded": (GObject.SignalFlags.RUN_FIRST, None, (Attachment,)),
     }
 
     STARTUP_SYNC_INIT_DURATION = 500
@@ -186,6 +189,32 @@ class SyncManager(GObject.Object):
         """
         return self.__sync_worker.check_endpoint_for_auth_token(endpoint, token)
 
+    def download_attachments(
+        self, attachments: dict[str, Attachment], on_thread: bool
+    ) -> Optional[bool]:
+        """Fetch attachments from server.
+
+        :param dict[str, Attachment] attachments: Attachments to transfer
+        :param bool on_thread: Whether to work on background thread
+        :return: Success when not working on thread
+        :rtype: bool, optional
+        """
+
+        def work(attachments: dict[str, Attachment]) -> bool:
+            had_failure = False
+            for attachment in attachments.values():
+                if not self.__download_single_attachment(attachment):
+                    had_failure = True
+            return not had_failure
+
+        if on_thread:
+            thread = Thread(target=work, args=(attachments,))
+            thread.daemon = True
+            thread.start()
+            return None
+        else:
+            return work(attachments)
+
     @GObject.Property(type=bool, default=False)
     def authenticated(self) -> bool:
         return self.__authenticated
@@ -222,11 +251,11 @@ class SyncManager(GObject.Object):
             return
 
         GLib.idle_add(self.emit, "started")
-        thread = Thread(target=self.__do_remote_sync)
+        thread = Thread(target=self.__sync_on_thread)
         thread.daemon = True
         thread.start()
 
-    def __do_remote_sync(self) -> None:
+    def __sync_on_thread(self) -> None:
         logging.debug("Refresh....")
 
         self.__lock.acquire()
@@ -476,6 +505,7 @@ class SyncManager(GObject.Object):
             for n in self.__note_manager.base_model
             if n.has_id and n.dirty and not n.locally_deleted and n.remote_id < 0
         ]
+
         for note in local_new:
             if not note.content_loaded:
                 self.__db.populate_note_content(note)
@@ -544,11 +574,8 @@ class SyncManager(GObject.Object):
             if deleted_note is not None:
                 self.emit("note-deleted", deleted_note)
                 remove_notes.append(deleted_note)
-                self.__db.delete_note(deleted_note.id)
         if len(remove_notes) > 0:
-            self.__note_manager.remove_notes_from_model(remove_notes)
-            for note in remove_notes:
-                self.__category_manager.note_deleted(note.category)
+            self.__note_manager.delete_notes(remove_notes, provide_undo=False)
 
     def __duplicate_note_for_sync_conflict(self, note: Note) -> None:
         if not note.content_loaded:
@@ -655,3 +682,30 @@ class SyncManager(GObject.Object):
                 break
 
         return matched
+
+    def __download_single_attachment(self, attachment: Attachment) -> bool:
+        self.__check_and_warn_for_other_note_attachment(attachment)
+        data = self.__sync_worker.get_attachment(attachment)
+        if not data:
+            logging.warning(f"Failed to download attachment '{attachment.path}'")
+            return False
+
+        if not self.__db.get_note_by_remote_id(attachment.note_remote_id):
+            logging.warning("Discarding attachment for removed note")
+            # Returning true as this isn't a failure
+            return True
+
+        if not write_attachment_to_disk(attachment, data):
+            return False
+
+        GLib.idle_add(self.emit, "attachment-downloaded", attachment)
+        return True
+
+    def __check_and_warn_for_other_note_attachment(self, attachment: Attachment) -> None:
+        path = attachment.path
+        this_note_prefix = f".attachments.{attachment.note_remote_id}/"
+        if path.startswith(".attachments.") and not path.startswith(this_note_prefix):
+            logging.warning(
+                f"Attachment '{attachment.filename}' appears to belong to another note. "
+                "This is unsupported; unpredictable behaviour ahead."
+            )
